local allowed = nil
local warned = {}
local lastAllowedIncomingPlayer = nil
local disabled = nil

local blockMessage = "ShutUp: Your message has been blocked, since you are not in my friends list or my guild."

local function ShouldShow(name)
	return allowed[name] or ShutUpDB[name]
end

function ShutUp_Disable() disabled = true end
function ShutUp_Enable() disabled = nil end

local function incomingFilter(self, event, text, player, _, _, _, gm)
	if not allowed or disabled then return end
	if type(gm) == "string" and (gm == "GM" or gm == "DEV") then return end
	if not ShouldShow(player) then
		if not warned[player] and type(ShutUpReply) == "string" then
			SendChatMessage(ShutUpReply, "WHISPER", nil, player)
			warned[player] = true
		end
		if lastAllowedIncomingPlayer then
			ChatEdit_SetLastTellTarget(lastAllowedIncomingPlayer)
		end
		return true
	else
		lastAllowedIncomingPlayer = player
	end
end

local function outgoingFilter(self, event, text, player, _, _, _, gm)
	if not allowed or disabled then return end
	if type(gm) == "string" and (gm == "GM" or gm == "DEV") then return end
	if type(ShutUpReply) == "string" and (text == ShutUpReply or text:find(ShutUpReply)) then
		return true
	end
	if not allowed[player] then
		ShutUpDB[player] = true
	end
end

ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", incomingFilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", outgoingFilter)

local function InsertFriends()
	if not allowed then allowed = {} end
	-- Find all friends
	for i = 1, GetNumFriends() do
		local name = GetFriendInfo(i)
		if name then allowed[name] = true end
	end
end

local function InsertGuildMembers()
	if not allowed then allowed = {} end
	-- Find all guild members.
	for i = 1, GetNumGuildMembers(true) do
		local name = GetGuildRosterInfo(i)
		if name then allowed[name] = true end
	end
end

local frame = CreateFrame("Frame")
frame:SetScript("OnEvent", function()
	if event == "PLAYER_LOGIN" then
		if type(ShutUpDB) ~= "table" then ShutUpDB = {} end
		if type(ShutUpReply) ~= "string" then ShutUpReply = blockMessage end

		if IsInGuild() then GuildRoster() end
		ShowFriends()
	elseif event == "PARTY_MEMBERS_CHANGED" then
		if not allowed then allowed = {} end
		for i = 1, GetNumPartyMembers() do
			local n = UnitName("party"..i)
			if n then allowed[n] = true end
		end
	elseif event == "RAID_ROSTER_UPDATE" then
		if not allowed then allowed = {} end
		for i = 1, GetNumRaidMembers() do
			local n = GetRaidRosterInfo(i)
			if n then allowed[n] = true end
		end
	elseif event == "FRIENDLIST_UPDATE" then
		InsertFriends()
	elseif event == "GUILD_ROSTER_UPDATE" then
		InsertGuildMembers()
	elseif event == "BN_CONNECTED" or event == "BN_FRIEND_INFO_CHANGED" then
		for i = 1, BNGetNumFriends() do
			local _, _, _, toonName, _, client = BNGetFriendInfo(i)
			if client == BNET_CLIENT_WOW and toonName then
				allowed[toonName] = true
			end
		end
	end
end)

frame:RegisterEvent("GUILD_ROSTER_UPDATE")
frame:RegisterEvent("FRIENDLIST_UPDATE")
frame:RegisterEvent("PLAYER_LOGIN")
frame:RegisterEvent("PARTY_MEMBERS_CHANGED")
frame:RegisterEvent("RAID_ROSTER_UPDATE")
frame:RegisterEvent("BN_CONNECTED")
frame:RegisterEvent("BN_FRIEND_INFO_CHANGED")

if SlashCmdList then
	SlashCmdList["ShutUp_SHORTHAND"] = function(text)
		if type(text) ~= "string" or text:trim():len() < 1 then
			ShutUpReply = nil
			print("ShutUp reply disabled.")
		else
			ShutUpReply = text
			print("ShutUp reply set to: '" .. tostring(text) .. "'.")
		end
	end
	SLASH_ShutUp_SHORTHAND1 = "/shutup"
end

